'use strict'

var express           = require('express')
var passport          = require('passport')
var bodyParser        = require('body-parser')
var cors              = require('cors')
var mongoose          = require('mongoose')
var mountAppRoutes    = require('./routes/app')
var mountAuthRoute    = require('./routes/auth')

class API {

  constructor(props, resources) {
    this.props = props
    this.resources = resources
    this.initialize()
  }

  initializeDB(callback) {
    mongoose.connect(this.props.dbUrl, callback);
  }

  initialize() {
    // Initialize the server
    this.app = express()
    this.app.use(bodyParser.json());
    this.app.use(bodyParser.urlencoded({extended: false}));
    this.app.use(cors());
    this.app.use(passport.initialize())
    this.app.use(passport.session())
  }

  start(callback) {
    var self = this
    this.initializeDB(function(error){
      if (error) {
        callback(error)
        return;
      }

      // Mount all custom application routes
      mountAppRoutes(self.app, self.resources, self.props)

      // Mount the authentication route
      mountAuthRoute(self.app, self.props)

      // Start the server
      self.app.listen(self.props.cloud.api.port, function () {
        callback()
      });
    });
  }

}

module.exports = API;
